@@ -104,6 +104,8 @@ class Arch_x86(Arch):
             # If we're likely to use KVM, request a full-featured CPU.
             # (NB: if KVM fails, this will cause problems.  We should probe.)
             ret.extend(["-cpu", "host"])  # We can't migrate regardless.
+        else:
+            ret.extend(["-machine", "q35"])
 
         return ret
 
@@ -181,7 +183,7 @@ class Arch_microvm(Arch_x86):
         ret = Arch.qemuargs(is_native, use_kvm)
 
         # Use microvm architecture for faster boot
-        ret.extend(["-M", "microvm,accel=kvm,pcie=on"])
+        ret.extend(["-M", "microvm,accel=kvm,pcie=on,rtc=on"])
 
         if is_native and use_kvm:
             # If we're likely to use KVM, request a full-featured CPU.
