@@ -98,6 +98,11 @@ def make_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="Give the guest read-write access to its root filesystem",
     )
+    g.add_argument(
+        "--gdb",
+        action="store_true",
+        help="Attach a gdb session to a running instance started with --debug",
+    )
     g.add_argument(
         "--graphics",
         action="store",
@@ -199,6 +204,12 @@ def make_parser() -> argparse.ArgumentParser:
         default=uname.machine,
         help="Guest architecture",
     )
+    g.add_argument(
+        "--cross-compile",
+        action="store",
+        metavar="CROSS_COMPILE_PREFIX",
+        help="Cross-compile compiler prefix",
+    )
     g.add_argument(
         "--busybox",
         action="store",
@@ -324,9 +335,10 @@ def has_memory_suffix(string):
 
 
 class Kernel:
-    __slots__ = ["kimg", "dtb", "modfiles", "moddir", "use_root_mods", "config"]
+    __slots__ = ["kimg", "version", "dtb", "modfiles", "moddir", "use_root_mods", "config"]
 
     kimg: str
+    version: str
     dtb: Optional[str]
     modfiles: List[str]
     moddir: Optional[str]
@@ -435,6 +447,8 @@ def find_kernel_and_mods(arch, args) -> Kernel:
                 + kimg
                 + " (modules may not work)\n"
             )
+        else:
+            kernel.version = kver
         kernel.kimg = kimg
         if args.mods == "none":
             kernel.modfiles = []
@@ -447,11 +461,6 @@ def find_kernel_and_mods(arch, args) -> Kernel:
             # the modules, just rely on /lib/modules in the target rootfs.
             if root_dir == "/" or args.root != '/':
                 kernel.use_root_mods = True
-            elif root_dir.startswith("/tmp"):
-                sys.stderr.write(
-                    "\nWarning: /tmp is hidden inside the guest, "
-                    + "kernel modules won't be supported at runtime unless you move them somewhere else.\n\n"
-                )
             kernel.moddir = f"{root_dir}/lib/modules/{kver}"
             if not os.path.exists(kernel.moddir):
                 kernel.modfiles = []
@@ -475,7 +484,7 @@ def find_kernel_and_mods(arch, args) -> Kernel:
     elif args.kdir is not None:
         kimg = os.path.join(args.kdir, arch.kimg_path())
         # Run get_kernel_version to check at least if the kernel image exist.
-        get_kernel_version(kimg)
+        kernel.version = get_kernel_version(kimg)
         kernel.kimg = kimg
         virtme_mods = os.path.join(args.kdir, ".virtme_mods")
         mod_file = os.path.join(args.kdir, "modules.order")
@@ -587,6 +596,7 @@ class VirtioFS:
             f"{self.guest_tools_path}/bin/virtiofsd",
             which("virtiofsd"),
             "/usr/libexec/virtiofsd",
+            "/usr/lib/virtiofsd/virtiofsd",
             "/usr/lib/virtiofsd",
             "/usr/lib/qemu/virtiofsd",
         )
@@ -786,6 +796,18 @@ def is_statically_linked(binary_path):
         return False
 
 
+def is_subpath(path, potential_parent):
+    # Normalize the paths to avoid issues with trailing slashes and different formats
+    path = os.path.abspath(path)
+    potential_parent = os.path.abspath(potential_parent)
+
+    # Find the common path
+    common_path = os.path.commonpath([path, potential_parent])
+
+    # Check if the common path is the same as the potential parent path
+    return common_path == potential_parent
+
+
 # Allowed characters in mount paths.  We can extend this over time if needed.
 _SAFE_PATH_PATTERN = "[a-zA-Z0-9_+ /.-]+"
 _RWDIR_RE = re.compile("^(%s)(?:=(%s))?$" % (_SAFE_PATH_PATTERN, _SAFE_PATH_PATTERN))
@@ -813,6 +835,18 @@ def do_it() -> int:
     if config.modfiles:
         need_initramfs = True
 
+    if args.gdb:
+        if kernel.version:
+            print(f"kernel version = {kernel.version}")
+        vmlinux = ''
+        if os.path.exists("vmlinux"):
+            vmlinux = "vmlinux"
+        elif os.path.exists(f"/usr/lib/debug/boot/vmlinux-{kernel.version}"):
+            vmlinux = f"/usr/lib/debug/boot/vmlinux-{kernel.version}"
+        command = ['gdb', '-q', '-ex', 'target remote localhost:1234', vmlinux]
+        os.execvp('gdb', command)
+        sys.exit(0)
+
     qemuargs: List[str] = [qemu.qemubin]
     kernelargs = []
 
@@ -977,6 +1011,19 @@ def do_it() -> int:
             if guestpath.startswith(".."):
                 arg_fail("%r is not inside the root" % hostpath)
 
+        # Check if paths are accessible both on the host and the guest.
+        if not os.path.exists(hostpath):
+            arg_fail(f"error: cannot access {hostpath} on the host")
+        # Guest path must be defined inside one of the overlays
+        guest_path_ok = False
+        for i, d in enumerate(args.overlay_rwdir):
+            if os.path.exists(guestpath) or is_subpath(guestpath, d):
+                guest_path_ok = True
+                break
+        if not guest_path_ok:
+            arg_fail(f"error: cannot initialize {guestpath} inside the guest " +
+                     "(path must be defined inside a valid overlay)")
+
         idx = mount_index
         mount_index += 1
         tag = "virtme.initmount%d" % idx
@@ -1363,6 +1410,8 @@ def do_it() -> int:
     if not args.verbose:
         kernelargs.append("quiet")
         kernelargs.append("loglevel=0")
+    else:
+        kernelargs.append("debug")
 
     # Now that we're done setting up kernelargs, append user-specified args
     # and then initargs
