@@ -10,6 +10,7 @@ import sys
 import socket
 import shutil
 import json
+import signal
 import tempfile
 from subprocess import (
     check_call,
@@ -77,7 +78,30 @@ def make_parser():
     """Main virtme-ng command line parser."""
 
     parser = argparse.ArgumentParser(
+        formatter_class=argparse.RawTextHelpFormatter,
         description="Build and run kernels inside a virtualized snapshot of your live system",
+        epilog="""\
+virtme-ng is a tool that allows to easily and quickly recompile and test a
+Linux kernel, starting from the source code. It allows to re‐ compile  the
+kernel in a few minutes (rather than hours), then the kernel is automatically
+started in a virtualized environment that is an exact copy-on-write copy of
+your live system, which means that any changes made to the virtualized
+environment do not affect the host system.
+
+In order to do this, a minimal config is produced (with the bare minimum
+support to test the kernel inside qemu), then the selected ker‐ nel is
+automatically built and started inside qemu, using the filesystem of the host
+as a copy-on-write snapshot.
+
+This means that you can safely destroy the entire filesystem, crash the kernel,
+etc. without affecting the host.
+
+NOTE: kernels produced with virtme-ng are lacking lots of features, in order to
+reduce the build time to the minimum and still provide you a usable kernel
+capable of running your tests and experiments.
+
+virtme-ng is based on virtme, written by Andy Lutomirski <luto@kernel.org>.
+""",
     )
     parser.add_argument(
         "--version", "-V", action="version", version=f"virtme-ng {VERSION}"
@@ -143,6 +167,12 @@ def make_parser():
         help="Fallback to the bash virtme-init (useful for debugging/development)",
     )
 
+    parser.add_argument(
+        "--gdb",
+        action="store_true",
+        help="Attach a debugging session to a running instance started with --debug",
+    )
+
     parser.add_argument(
         "--snaps", action="store_true", help="Allow to execute snaps inside virtme-ng"
     )
@@ -183,10 +213,18 @@ def make_parser():
         "to override default config settings",
     )
 
+    parser.add_argument(
+        "--configitem",
+        action="append",
+        help="add a CONFIG_ITEM=val, after --config <fragments>, "
+        "these override previous config settings",
+    )
+
     parser.add_argument(
         "--compiler",
         action="store",
-        help="Compiler to be used as CC when building the kernel",
+        help="[deprecated] Compiler to be used as CC when building the kernel. "
+        "Please set CC= and HOSTCC= variables in the virtme-ng command line instead.",
     )
 
     parser.add_argument(
@@ -404,6 +442,12 @@ def make_parser():
         "(default is host architecture)",
     )
 
+    parser.add_argument(
+        "--cross-compile",
+        action="store",
+        help="Set cross-compile prefix"
+    )
+
     parser.add_argument(
         "--force",
         action="store_true",
@@ -575,11 +619,14 @@ class KernelSource:
             arch = ARCH_MAPPING[arch]["qemu_name"]
             cmd += f" --arch {arch}"
         user_config = str(Path.home()) + "/.config/virtme-ng/kernel.config"
+        if os.path.exists(user_config):
+            cmd += f" --custom {user_config}"
         if args.config:
             for conf in args.config:
                 cmd += f" --custom {conf}"
-        if os.path.exists(user_config):
-            cmd += f" --custom {user_config}"
+        if args.configitem:
+            for citem in args.configitem:
+                cmd += f" --configitem {citem}"
         # Propagate additional Makefile variables
         for var in args.envs:
             cmd += f" {var} "
@@ -692,6 +739,9 @@ class KernelSource:
                 arg_fail(f"unsupported architecture: {arch}")
             target = ARCH_MAPPING[arch]["kernel_target"]
             cross_compile = ARCH_MAPPING[arch]["cross_compile"]
+            if args.cross_compile != "":
+                cross_compile = args.cross_compile
+
             cross_arch = ARCH_MAPPING[arch]["linux_name"]
         else:
             target = "bzImage"
@@ -699,7 +749,7 @@ class KernelSource:
             cross_arch = None
         make_command = MAKE_COMMAND
         if args.compiler:
-            make_command += f" CC={args.compiler}"
+            make_command += f" HOSTCC={args.compiler} CC={args.compiler}"
         if args.skip_modules:
             make_command += f" {target}"
         if cross_compile and cross_arch:
@@ -798,7 +848,7 @@ class KernelSource:
         else:
             self.virtme_param["overlay_rwdir"] = " ".join(
                 f"--overlay-rwdir {d}"
-                for d in ("/etc", "/lib", "/home", "/opt", "/srv", "/usr", "/var")
+                for d in ("/etc", "/lib", "/home", "/opt", "/srv", "/usr", "/var", "/tmp")
             )
         # Add user-specified overlays.
         for item in args.overlay_rwdir:
@@ -904,15 +954,15 @@ class KernelSource:
             self.virtme_param["verbose"] = ""
 
     def _get_virtme_append(self, args):
+        append = []
         if args.append is not None:
-            append = []
             for item in args.append:
                 split_items = item.split()
                 for split_item in split_items:
                     append.append("-a " + split_item)
-            self.virtme_param["append"] = " ".join(append)
-        else:
-            self.virtme_param["append"] = ""
+        if args.debug:
+            append.append("-a nokaslr")
+        self.virtme_param["append"] = " ".join(append)
 
     def _get_virtme_memory(self, args):
         if args.memory is None:
@@ -933,6 +983,15 @@ class KernelSource:
         else:
             self.virtme_param["balloon"] = ""
 
+    def _get_virtme_gdb(self, args):
+        if args.gdb:
+            def signal_handler(_signum, _frame):
+                pass  # No action needed for SIGINT in child (gdb will handle)
+            signal.signal(signal.SIGINT, signal_handler)
+            self.virtme_param["gdb"] = "--gdb"
+        else:
+            self.virtme_param["gdb"] = ""
+
     def _get_virtme_snaps(self, args):
         if args.snaps:
             self.virtme_param["snaps"] = "--snaps"
@@ -1002,6 +1061,7 @@ class KernelSource:
         self._get_virtme_memory(args)
         self._get_virtme_numa(args)
         self._get_virtme_balloon(args)
+        self._get_virtme_gdb(args)
         self._get_virtme_snaps(args)
         self._get_virtme_busybox(args)
         self._get_virtme_qemu(args)
@@ -1038,6 +1098,7 @@ class KernelSource:
             + f'{self.virtme_param["memory"]} '
             + f'{self.virtme_param["numa"]} '
             + f'{self.virtme_param["balloon"]} '
+            + f'{self.virtme_param["gdb"]} '
             + f'{self.virtme_param["snaps"]} '
             + f'{self.virtme_param["busybox"]} '
             + f'{self.virtme_param["qemu"]} '
